package com.ti.mobpo.ui.pokesearch

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ti.mobpo.data.Favourite
import com.ti.mobpo.data.FavouritesRepository
import com.ti.mobpo.model.PokemonDetails
import com.ti.mobpo.network.PokeApi
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import java.io.IOException



class PokeSearchViewModel(private val favouritesRepository: FavouritesRepository) : ViewModel() {
    private val service = PokeApi.retrofitService;

    private val _pokemonDetails = MutableStateFlow<List<PokemonDetails>?>(null)
    val pokemonDetails: StateFlow<List<PokemonDetails>?> = _pokemonDetails.asStateFlow()

    fun search(query: String) {
        viewModelScope.launch {
            try {
                val response = service.getPokemon()
                val filteredList = response.results.filter { it.name.contains(query, ignoreCase = true) }
                val detailsList = mutableListOf<PokemonDetails>()
                for (pokemonSpecies in filteredList) {
                    val details = service.getPokemonDetails(extractPokemonId(pokemonSpecies.url))
                    val isFavorite = favouritesRepository.isFavourite(details.id)
                    detailsList.add(details.copy(isFavorite = isFavorite))
                }
                _pokemonDetails.value = detailsList
            } catch (e: IOException) {
                /* Handle error */
            }
        }
    }

    private fun extractPokemonId(url: String): Int {
        val parts = url.split("/")
        return parts[parts.size - 2].toInt()
    }

    fun toggleFavorite(pokemonId: Int) {
        _pokemonDetails.value = _pokemonDetails.value?.map { pokemon ->
            if (pokemon.id == pokemonId) {
                pokemon.copy(isFavorite = !pokemon.isFavorite)
            } else {
                pokemon
            }
        }
        viewModelScope.launch {
            if (_pokemonDetails.value != null) {
                val pokemon = _pokemonDetails.value!!.find { it.id == pokemonId }
                pokemon?.let {
                    if (it.isFavorite) {
                        favouritesRepository.insertItem(Favourite(it.id, it.name))
                    } else {
                        favouritesRepository.deleteItem(Favourite(it.id, it.name))
                    }
                }
            }
        }
    }
}
